;; Generated package description from realgud-trepan-ni.el
(define-package "realgud-trepan-ni" "1.0.0" "Realgud front-end to trepan-ni" '((load-relative "1.2") (realgud "1.4.5") (cl-lib "0.5") (emacs "25")) :url "http://github.com/realgud/realgud-node-inspect" :keywords nil)
